<?php

namespace Drupal\virtual_base\EventSubscriber;

use Drupal\Component\Utility\Html;
use Drupal\Core\Render\HtmlResponse;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Defines a test session subscriber that checks whether the session is empty.
 *
 *  @SuppressWarnings(PHPMD.CamelCaseParameterName)
 *  @SuppressWarnings(PHPMD.CamelCaseVariableName)
 *  @SuppressWarnings(PHPMD.LongClassName)
 *  @SuppressWarnings(PHPMD.LongVariable)
 */
class VirtualBaseSubscriber implements EventSubscriberInterface {

  /**
   * Callback kernel event on respond.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The Event to process.
   */
  public function onRespond(ResponseEvent $event) {
    $response = $event->getResponse();

    if (!$response instanceof HtmlResponse) {
      return;
    }

    $request = $event->getRequest();
    // No virtual base feature found.
    if (!$request->query->has('_virtual_base')) {
      return;
    }
    // @todo isn't better to get the config instead of the request which can
    // be modify by attack maybe?
    $path_prefix = $request->query->get('_virtual_base');
    // Get all the html content.
    $text = $response->getContent();
    $html_dom = Html::load($text);
    $somethingDone = FALSE;
    // $xpath = new \DOMXPath($html_dom);
    // foreach ($xpath->query('//' . $allowed_tag . '[@*]') as $element) {
    // $this->filterElementAttributes($element, $allowed_attributes);
    // }
    // Get all links inside html and transform the href.
    $this->changeAttributeTag($html_dom, $somethingDone, $path_prefix, 'a', 'href');
    $this->changeAttributeTag($html_dom, $somethingDone, $path_prefix, 'link', 'href');
    // Get all image inside html and transform the src.
    $this->changeAttributeTag($html_dom, $somethingDone, $path_prefix, 'img', 'src');
    $this->changeAttributeTag($html_dom, $somethingDone, $path_prefix, 'svg', 'src');

    // Get all script inside html and transform the src.
    $this->changeAttributeTag($html_dom, $somethingDone, $path_prefix, 'script', 'src');

    // Nothing change inside the html.
    if (!$somethingDone) {
      return;
    }

    $text2 = Html::serialize($html_dom);
    $response->setContent($text2);
  }

  /**
   * Rewrite attribute of an html tag with the virtual base prefix.
   *
   * @param \DOMDocument $html_dom
   *   DOM document.
   * @param bool $change_dom
   *   Change DOM?
   * @param string $path_prefix
   *   Path prefix.
   * @param string $tag_name
   *   Tag name.
   * @param string $attribute_name
   *   Attribute name.
   *
   * @return void
   *   Void.
   */
  private function changeAttributeTag(\DOMDocument $html_dom, bool &$change_dom, string $path_prefix, string $tag_name, string $attribute_name) : void {
    $tags = $html_dom->getElementsByTagName($tag_name);
    foreach ($tags as $tag) {
      if (!$tag->hasAttribute($attribute_name)) {
        continue;
      }
      $attribute = $tag->getAttribute($attribute_name);
      if (str_starts_with($attribute, '/') && !str_starts_with($attribute, $path_prefix)) {
        $tag->setAttribute($attribute_name, $path_prefix . $attribute);
        $change_dom = TRUE;
      }
    }
  }

  /**
   * Registers the methods in this class that should be listeners.
   *
   * @return array
   *   An array of event listener definitions.
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::RESPONSE][] = ['onRespond', -9999];
    return $events;
  }

}
